unit GridEhEd;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ToolWin, {ImgList,} Menus, DBGridEH, DBGrids,
  DsgnIntf, DsgnWnds, LibIntf;

type
  TDBGridEHColumnsEditor = class(TDesignWindow)
    ToolBar1: TToolBar;
    tbAddNew: TToolButton;
    tbDeleteSelected: TToolButton;
    ToolButton3: TToolButton;
    tbAddAllFields: TToolButton;
    tbRestoreDefaults: TToolButton;
    ImageList1: TImageList;
    PopupMenu1: TPopupMenu;
    miAddNew: TMenuItem;
    miDeleteSelected: TMenuItem;
    miAddAllFields: TMenuItem;
    miRestoreDefaults: TMenuItem;
    miSelectAll: TMenuItem;
    N1: TMenuItem;
    ListView1: TListView;
    tbMoveUp: TToolButton;
    tbMoveDown: TToolButton;
    ToolButton4: TToolButton;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure tbAddNewClick(Sender: TObject);
    procedure tbDeleteSelectedClick(Sender: TObject);
    procedure tbAddAllFieldsClick(Sender: TObject);
    procedure tbRestoreDefaultsClick(Sender: TObject);
    procedure ListView1Change(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure FormResize(Sender: TObject);
    procedure miAddNewClick(Sender: TObject);
    procedure miDeleteSelectedClick(Sender: TObject);
    procedure miSelectAllClick(Sender: TObject);
    procedure miAddAllFieldsClick(Sender: TObject);
    procedure miRestoreDefaultsClick(Sender: TObject);
    procedure tbMoveUpClick(Sender: TObject);
    procedure tbMoveDownClick(Sender: TObject);
  private
    FDBGridEH:TDBGridEH;
    function CheckCollection: Boolean;
    procedure SetDBGridEH(const Value: TDBGridEH);
    procedure UpdateData;
    procedure Copy;
    procedure Cut;
    procedure Paste;
  protected
    function UniqueName(Component: TComponent): string; override;
    procedure Activated; override;
    { Private declarations }
  public
    procedure EditAction(Action: TEditAction); override;
    procedure FormModified; override;
    procedure FormClosed(Form: TCustomForm); override;
    function GetEditState: TEditState; override;
{$IFDEF VER120} {Borland Delphi 4.0 }
    procedure ComponentDeleted(Component: IPersistent); override;
{$ELSE}
    procedure ComponentDeleted(Component: TComponent); override;
{$ENDIF}

    property DBGridEH: TDBGridEH read FDBGridEH
         write SetDBGridEH;
    { Public declarations }
  end;


{ TDBGridEHColumnsProperty }

  TDBGridEHColumnsProperty = class(TPropertyEditor)
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure Edit; override;
  end;

{ TDBGridEHEditor }

  TDBGridEHEditor = class(TComponentEditor)
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

var
  DBGridEHColumnsEditor: TDBGridEHColumnsEditor;

//procedure ShowItemsEditor({Designer: TDesigner;}
//  ADBGridEH: TDBGridEH);

implementation

{$R *.DFM}

{$IFDEF VER120} {Borland Delphi 4.0 }
type
  TDesigner = IDesigner;
  TFormDesigner = IFormDesigner;
{$ENDIF}

function FindEditor(ADBGridEH: TDBGridEH): TDBGridEHColumnsEditor;
var
  I: Integer;
begin
  Result := nil;
  for I := 0 to Screen.FormCount - 1 do begin
    if Screen.Forms[I] is TDBGridEHColumnsEditor then begin
      if TDBGridEHColumnsEditor(Screen.Forms[I]).DBGridEH = ADBGridEH then
      begin
        Result := TDBGridEHColumnsEditor(Screen.Forms[I]);
        Break;
      end;
    end;
  end;
end;

procedure ShowItemsEditor(Designer: TDesigner;
  ADBGridEH: TDBGridEH);
var
  Editor: TDBGridEHColumnsEditor;
begin
  if ADBGridEH = nil then Exit;
  Editor := FindEditor(ADBGridEH);
  if Editor = nil then begin
    Editor := TDBGridEHColumnsEditor.Create(Application);
    try
      Editor.Designer := TFormDesigner(Designer);
      Editor.DBGridEH := ADBGridEH;
      Editor.Show;
    except
      Editor.Free;
      raise;
    end;
  end
  else begin
    Editor.Show;
    if Editor.WindowState = wsMinimized then
      Editor.WindowState := wsNormal;
  end;
end;


{ TDBGridColumnsEditor }

function TDBGridEHColumnsEditor.CheckCollection: Boolean;
begin
  Result := (DBGridEH <> nil) and (DBGridEH.Columns <> nil)
    and (Designer.Form <> nil);
end;

procedure TDBGridEHColumnsEditor.SetDBGridEH(const Value: TDBGridEH);
begin
  if FDBGridEH <> Value then begin
    FDBGridEH := Value;
    UpdateData;
  end;
end;

procedure TDBGridEHColumnsEditor.UpdateData;
var
  Empty,Modified: Boolean;
  i:Integer;
  ListItem: TListItem;
begin
  if CheckCollection then begin
    Caption := Format('Edition %s.Columns', [DBGridEH.Name]);
    Empty := DBGridEH.Columns.Count = 0;
  end
  else Empty := True;
  if Empty then begin
    ListView1.Items.Clear;
  end
  else begin
    Modified := False;
    if (DBGridEH.Columns.Count <> ListView1.Items.Count) then Modified := True;
    if (Modified = False) then
      for i := 0 to DBGridEH.Columns.Count - 1 do
        if (ListView1.Items[i].Data <> DBGridEH.Columns[i]) then begin
          Modified := True;
          Break;
        end;
    if (Modified = True) then begin
      ListView1.Items.BeginUpdate;
      ListView1.Items.Clear;
      try
        if (DBGridEH.Columns.State = csCustomized) then begin
           for i := 0 to DBGridEH.Columns.Count - 1 do begin
             ListItem := ListView1.Items.Add;
             ListItem.Caption := IntToStr(i) + ' - ' + DBGridEH.Columns[i].DisplayName;
             ListItem.Data := DBGridEH.Columns[i];
           end;
        end;
      finally
        ListView1.Items.EndUpdate;
      end;
    end else //      FieldName
      for i := 0 to DBGridEH.Columns.Count - 1 do ListView1.Items[i].Caption := IntToStr(i) + ' - ' + DBGridEH.Columns[i].DisplayName;
  end;
  ListView1Change(nil,nil,ctState);
end;

procedure TDBGridEHColumnsEditor.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TDBGridEHColumnsEditor.FormCreate(Sender: TObject);
begin
  DBGridEH := nil;
  if NewStyleControls then Font.Style := [];
end;

procedure TDBGridEHColumnsEditor.tbAddNewClick(Sender: TObject);
begin
  DBGridEH.Columns.Add;
  UpdateData;
  Designer.Modified;
  ListView1.Items[ListView1.Items.Count-1].Selected := True;
  ListView1Change(nil,nil,ctState);
end;

procedure TDBGridEHColumnsEditor.tbDeleteSelectedClick(Sender: TObject);
var i,sel:Integer;
//  ListItem: TListItem;
  FComponents: TComponentList;
begin
  sel := -1;
  if (ListView1.SelCount > 0) then begin
    FComponents := TComponentList.Create;
    try
      DBGridEH.Columns.BeginUpdate;
      ListView1.Items.BeginUpdate;
      sel := ListView1.Items.IndexOf(ListView1.Selected);
      for i := ListView1.Items.Count - 1 downto 0 do
       if (ListView1.Items[i].Selected = True) and Assigned(ListView1.Items[i].Data) then begin
         FComponents.Add(TColumnEH(ListView1.Items[i].Data));
       end;
      ListView1.Items.Clear;

      for i := 0 to FComponents.Count - 1 do FComponents[i].Free;

    finally
      ListView1.Items.EndUpdate;
      DBGridEH.Columns.EndUpdate;
      FComponents.Free;
      Designer.Modified;
      if (ListView1.Items.Count > 0) then
        if (sel > ListView1.Items.Count - 1) then
           ListView1.Items[ListView1.Items.Count-1].Selected := True
        else
           ListView1.Items[sel].Selected := True;
      ListView1Change(nil,nil,ctState);
    end;
   end;
end;

procedure TDBGridEHColumnsEditor.tbAddAllFieldsClick(Sender: TObject);
var msgValue:Word;
    i:Integer;
    Col:TColumnEH;
begin
  if (DBGridEH.Columns.State = csDefault) then
    DBGridEH.Columns.State := csCustomized
  else begin
   if (DBGridEH.Columns.Count > 0) then begin
     msgValue := MessageDlg('Delete existing columns?',
                      mtConfirmation, [mbYes, mbNo, mbCancel], 0);
     case msgValue of
       mrYes: DBGridEH.Columns.Clear;
       mrCancel: Exit;
     end;
   end;
   for i := 0 to DBGridEH.DataSource.DataSet.FieldCount - 1 do begin
     Col := DBGridEH.Columns.Add;
     Col.FieldName := DBGridEH.DataSource.DataSet.Fields[i].FieldName;
   end;
  end;
  UpdateData;
  Designer.Modified;
end;

procedure TDBGridEHColumnsEditor.tbRestoreDefaultsClick(Sender: TObject);
var i:Integer;
  ListItem: TListItem;
begin
  if (ListView1.SelCount > 0) then begin
    ListItem := ListView1.Selected;
    for i := 0 to ListView1.SelCount - 1 do begin
      TColumnEH(ListItem.Data).RestoreDefaults;
      ListItem := ListView1.GetNextItem(ListItem,sdBelow,[isSelected]);
    end;
    ListView1Change(nil,nil,ctState);
    Designer.Modified;
  end;
end;

procedure TDBGridEHColumnsEditor.ListView1Change(Sender: TObject;
  Item: TListItem; Change: TItemChange);
var
  FComponents: TComponentList;
  i: Integer;
//  ListItem: TListItem;
begin
  if (Change <> ctState)  then Exit;
  tbDeleteSelected.Enabled := ListView1.SelCount > 0;
  tbRestoreDefaults.Enabled := ListView1.Items.Count > 0;

  miDeleteSelected.Enabled := ListView1.SelCount > 0;
  miRestoreDefaults.Enabled := ListView1.Items.Count > 0;
  miSelectAll.Enabled := ListView1.Items.Count > 0;
  tbMoveDown.Enabled := ListView1.Items.Count > 0;
  tbMoveUp.Enabled := ListView1.Items.Count > 0;
  tbAddAllFields.Enabled := Assigned(DBGridEH) and Assigned(DBGridEH.DataSource) and
    Assigned(DBGridEH.Datasource.Dataset) and (DBGridEH.Datasource.Dataset.FieldCount > 0);

  if CheckCollection and Active then begin
     FComponents := TComponentList.Create;
    if (ListView1.SelCount > 0) then begin
      for i := ListView1.Items.Count - 1 downto 0 do
       if (ListView1.Items[i].Selected = True) and Assigned(ListView1.Items[i].Data) then
         FComponents.Add(TColumnEH(ListView1.Items[i].Data));
    end
    else FComponents.Add(DBGridEH.Columns);
    SetSelection(FComponents);
  end;
end;

procedure TDBGridEHColumnsEditor.Activated;
begin
  ListView1Change(nil,nil,ctState);
end;

function TDBGridEHColumnsEditor.UniqueName(Component: TComponent): string;
var
  Temp: string;
begin
  if (Component <> nil) then Temp := Component.ClassName
  else Temp := TColumnEH.ClassName;
  if (UpCase(Temp[1]) = 'T') and (Length(Temp) > 1) then
    System.Delete(Temp, 1, 1);
  Result := Designer.UniqueName(Temp);
end;

procedure TDBGridEHColumnsEditor.EditAction(Action: TEditAction);
begin
  case Action of
    eaCut: Cut;
    eaCopy: Copy;
    eaPaste: Paste;
    eaDelete: tbDeleteSelectedClick(Self);
  end;
end;

procedure TDBGridEHColumnsEditor.Copy;
var
  CompList: TComponentList;
  ListItem: TListItem;
  i:Integer;
begin
  CompList := TComponentList.Create;
  try
    if (ListView1.SelCount > 0) then begin
      ListItem := ListView1.Selected;
      for i := 0 to ListView1.SelCount - 1 do begin
        CompList.Add(TColumnEH(ListItem.Data));
        ListItem := ListView1.GetNextItem(ListItem,sdBelow,[isSelected]);
      end;
    end;
//    CopyComponents(DBGridEH.Columns, CompList);
  finally
    CompList.Free;
  end;
end;

procedure TDBGridEHColumnsEditor.Cut;
begin
  Copy;
  tbDeleteSelectedClick(Self);
end;

procedure TDBGridEHColumnsEditor.Paste;
var
  CompList: TComponentList;
begin
  if CheckCollection then begin
    CompList := TComponentList.Create;
    try
//      PasteComponents(DBGridEH.Columns, DBGridEH.Columns, CompList);
      UpdateData;
    finally
      CompList.Free;
    end;
  end;
end;

procedure TDBGridEHColumnsEditor.FormModified;
begin
  if not (csDestroying in ComponentState) then UpdateData;
end;

procedure TDBGridEHColumnsEditor.FormClosed(Form: TCustomForm);
begin
  if Form = Designer.Form then Close;
end;

{$IFDEF VER120} {Borland Delphi 4.0 }
procedure TDBGridEHColumnsEditor.ComponentDeleted(Component: IPersistent);
begin
  if ExtractPersistent(Component) = DBGridEH then begin
{$ELSE}
procedure TDBGridEHColumnsEditor.ComponentDeleted(Component: TComponent);
begin
  if Component = DBGridEH then begin
{$ENDIF}
    DBGridEH := nil;
    Close;
  end;
end;

function TDBGridEHColumnsEditor.GetEditState: TEditState;
begin
  Result := [];
  if tbDeleteSelected.Enabled then Result := [esCanDelete];
//  if tbDeleteSelected.Enabled then Result := [esCanDelete, esCanCut, esCanCopy];
//  if ClipboardComponents then Include(Result, esCanPaste);
end;

{ TDBGridEHColumnsProperty }

procedure TDBGridEHColumnsProperty.Edit;
begin
  ShowItemsEditor(Designer, TDBGridEH(GetComponent(0)));
end;

function TDBGridEHColumnsProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

function TDBGridEHColumnsProperty.GetValue: string;
begin
  FmtStr(Result, '(%s)', [GetPropType^.Name]);
end;


{ TDBGridEHEditor }

procedure TDBGridEHEditor.ExecuteVerb(Index: Integer);
begin
  case Index of
    0: ShowItemsEditor(Designer, TDBGridEH(Component));
  end;
end;

function TDBGridEHEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := 'Columns Editor ...';
  end;
end;

function TDBGridEHEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

procedure TDBGridEHColumnsEditor.FormResize(Sender: TObject);
begin
  ListView1.Columns[0].Width := ListView1.ClientWidth;
end;

procedure TDBGridEHColumnsEditor.miAddNewClick(Sender: TObject);
begin
  tbAddNewClick(Sender);
end;

procedure TDBGridEHColumnsEditor.miDeleteSelectedClick(Sender: TObject);
begin
  tbDeleteSelectedClick(Sender);
end;

procedure TDBGridEHColumnsEditor.miSelectAllClick(Sender: TObject);
var i:Integer;
begin
  for i := 0 to ListView1.Items.Count - 1 do ListView1.Items[i].Selected := True;
end;

procedure TDBGridEHColumnsEditor.miAddAllFieldsClick(Sender: TObject);
begin
  tbAddAllFieldsClick(Sender);
end;

procedure TDBGridEHColumnsEditor.miRestoreDefaultsClick(Sender: TObject);
begin
  tbRestoreDefaultsClick(Sender);
end;

procedure TDBGridEHColumnsEditor.tbMoveUpClick(Sender: TObject);
var i:Integer;
    idxs: TList;
begin
  idxs := TList.Create;
  try
    DBGridEH.Columns.BeginUpdate;
    try
      for i := 1 to ListView1.Items.Count - 1 do
        if (ListView1.Items[i].Selected = True) then
          idxs.Add(Pointer(i));
      for i := 1 to ListView1.Items.Count - 1 do
        if (ListView1.Items[i].Selected = True) then
          DBGridEH.Columns[i].Index := i-1;
    finally
      DBGridEH.Columns.EndUpdate;
    end;

    ListView1.Items.BeginUpdate;
    try
      UpdateData;
      for i := 0 to idxs.Count - 1 do
        ListView1.Items[Integer(idxs.Items[i])-1].Selected := True;
     finally
       ListView1.Items.EndUpdate;
     end;
  finally
    idxs.Free;
  end;
end;

procedure TDBGridEHColumnsEditor.tbMoveDownClick(Sender: TObject);
var i:Integer;
    idxs: TList;
begin
  idxs := TList.Create;
  try
    DBGridEH.Columns.BeginUpdate;
    try
      for i := 0 to ListView1.Items.Count - 2 do
        if (ListView1.Items[i].Selected = True) then
          idxs.Add(Pointer(i));
      for i := ListView1.Items.Count - 2 downto 0 do
        if (ListView1.Items[i].Selected = True) then
          DBGridEH.Columns[i].Index := i+1;
    finally
      DBGridEH.Columns.EndUpdate;
    end;

    ListView1.Items.BeginUpdate;
    try
      UpdateData;
      for i := 0 to idxs.Count - 1 do
        ListView1.Items[Integer(idxs.Items[i])+1].Selected := True;
     finally
       ListView1.Items.EndUpdate;
     end;
  finally
    idxs.Free;
  end;
end;

end.
